from __future__ import with_statement
import re
from pprint import pprint


class APIParser(object):
    re_strip_desc = re.compile(ur"^([^\ ]+) -", re.I )
    re_header = re.compile(ur"^(.+) functions$", re.I)

    chunk_pattern = r"""
        {	name = 'support.function.wow.%s';
            match = '(?<![^.]\.|:)\b(%s)\b(?=[( {])';
        },
    """

    def __init__(self, filename, output):
        self.filename = filename
        self.output = output
    
    @property
    def raw_data(self):
        with open(self.filename) as fp:
            return fp.readlines()
    
    @property
    def data(self):
        raw = self.raw_data
        cleaned = []

        for line in raw:
            line = line.strip()

            if not line or line.endswith('luaapi'): 
                continue
            
            if ' - ' not in line and self.re_header.match(line):
                cleaned.append(line)
                continue
            
            cleaned.append(self.re_strip_desc.match(line).groups()[0])
        
        return cleaned
    
    
    def process(self):
        chunks = {}
        
        header, chunk = None, None
        
        for line in self.data:
            header_check = self.re_header.match(line)
            if header_check:
                #put up the last header
                if header:
                    chunks[header] = chunk
                
                header = header_check.groups()[0].lower().replace(u'-', u'').replace(u'related', u'').replace(u'information', u'').split(u'/')[0].replace(u' ', u'')
                chunk = []
                continue
            chunk.append(line)
        
        chunks[header] = chunk
            
        with open(self.output, 'w') as fp:
            for header, chunk in chunks.iteritems():
                fp.write(self.chunk_pattern % (header, u'|'.join(chunk)))
                
class EventParser(object):
    
    re_strip_desc = re.compile(ur'^([A-Z_]+)\s.+')
    chunk_pattern = r"""
        {	name = 'support.constant.wow.events';
        	match = '(''|")(%s)\1';
        },
    """
    
    def __init__(self, filename, output):
        self.filename = filename
        self.output = output

    @property
    def raw_data(self):
        with open(self.filename) as fp:
            return fp.readlines()

    @property
    def data(self):
        raw = self.raw_data
        cleaned = []

        for line in raw:
            line = line.strip()
            if not line: 
                continue
            
            match = self.re_strip_desc.match(line)
            
            if match:
                cleaned.append(match.groups()[0])

        return cleaned
    
    def process(self):
        with open(self.output, 'w') as fp:
            fp.write(self.chunk_pattern % u'|'.join(self.data))
            

class WidgetParser(object):
    re_strip_desc = re.compile(ur"^([^\ ]+) ", re.I )
    re_parse = re.compile(ur"^([^:]+):([^\(]+)", re.I)

    chunk_pattern = r"""
        {	name = 'support.function.wow.widget.%s';
            match = '(?<!=[.:])\s*\b(%s)\b(?=[( {])';
        },
    """

    hierarchy = {
        'process_order': ['PlayerModel', 'Model', 'Button', 'Frame', 'LayeredRegion', 'VisibleRegion', 'Region', 'FontInstance', 'UIObject'],

        'UIObject': ['Region', 'FontInstance'],
            'FontInstance': ['FontString', 'Font', 'EditBox', 'MessageFrame', 'ScrollingMessageFrame'],    
            'Region': ['VisibleRegion'],
                'VisibleRegion': ['LayeredRegion', 'Frame'],
                    'LayeredRegion': ['Texture', 'FontString'], 
                    'Frame': ['MovieFrame', 'Minimap', 'GameTooltip', 'MessageFrame', 'Slider', 'SimpleHTML', 'ColorSelect', 'Button', 'ScrollFrame', 'Model', 'ScrollingMessageFrame', 'EditBox', 'StatusBar', 'Cooldown'], 
                        'Button': ['CheckButton'], 
                        'Model': ['PlayerModel'], 
                            'PlayerModel': ['TabardModel', 'DressUpModel'],


    }

    def __init__(self, filename, output):
        self.filename = filename
        self.output = output

    @property
    def raw_data(self):
        with open(self.filename) as fp:
            return fp.readlines()

    @property
    def data(self):
        raw = self.raw_data
        cleaned = []

        for line in raw:
            line = line.strip()

            if not line: 
                continue
            
            match = self.re_strip_desc.match(line)
            if match:
                cleaned.append(match.groups()[0])

        return cleaned
        
    def clean_chunks(self):
        chunk_sets = dict((header, set(funcs)) for header, funcs in self.chunks.items())
        new_chunks = dict((header, set(funcs)) for header, funcs in self.chunks.items())
        
#        self.hierarchy['process_order'].reverse()
        for header in self.hierarchy['process_order']:
            children = self.hierarchy[header]
            
            #get common funcs
            common = chunk_sets[header].copy()
            for child in children:
                common.intersection_update(chunk_sets[child])
            
            #
            for child in children:
                new_chunks[child].difference_update(common)
            
            diff = common == chunk_sets[header]
            print header
            if not diff: 
                print 'fuck'
                print header, chunk_sets[header].difference(common)
                
     
        self.chunks = new_chunks

    def process(self):
        chunks = {}

        for line in self.data:
            match = self.re_parse.match(line.strip())
            
            if not match:
                print line
                continue
            
            header, func = match.groups()
            
            if header not in chunks:
                chunks[header] = []
                
            chunks[header].append(func)
            
        
        self.chunks = chunks
        self.clean_chunks()
        
        with open(self.output, 'w') as fp:
            for header, chunk in self.chunks.iteritems():
                fp.write(self.chunk_pattern % (header, u'|'.join(chunk)))
    

if __name__ == '__main__':
    APIParser('raw_api', 'api.chunk').process()
    EventParser('raw_events', 'events.chunk').process()
    WidgetParser('raw_widget', 'widget.chunk').process()
    

    